@@ -23,14 +23,17 @@ try:
     from lauetools import IOLaueTools as IOLT
     from lauetools import CrystalParameters as CP
     from lauetools import lauecore as LT
+    from lauetools import readmccd as RMCCD
+
 except:
     import lauetoolsnn.lauetools.imageprocessing as ImProc
     import lauetoolsnn.lauetools.LaueGeometry as Lgeo
     import lauetoolsnn.lauetools.IOLaueTools as IOLT
     import lauetoolsnn.lauetools.CrystalParameters as CP
     import lauetoolsnn.lauetools.lauecore as LT
+    import lauetoolsnn.lauetools.readmccd as RMCCD
 
-from skimage.feature import blob_dog, blob_log
+from skimage.feature import blob_log #, blob_dog
 import cv2
 from skimage import morphology
 
@@ -39,6 +42,8 @@ def process_images(filename, file_directory):
     
     ##Idea is to get the XY pixel from the oreintation matrix and extract all
     ##closest peaks to a given ori mat.
+    # original_data = plt.imread(filename)
+    
     data_8bit_raw = plt.imread(filename)
     framedim = data_8bit_raw.shape
     CCDLabel = "sCMOS_16M"
@@ -76,10 +81,11 @@ def process_images(filename, file_directory):
     indsort = np.argsort(s_intensity)[::-1]
     s_posx=np.take(s_posx, indsort)
     s_posy=np.take(s_posy, indsort)
+    s_intensity=np.take(s_intensity, indsort)
     
-    blobs_log_all_x, blobs_dog_all_x = [], []
-    blobs_log_all_y, blobs_dog_all_y = [], []
-    blobs_log_all_ecc, blobs_dog_all_ecc = [], []
+    blobs_log_all_x= []#, blobs_dog_all_x = [], []
+    blobs_log_all_y= []#, blobs_dog_all_y = [], []
+    blobs_log_all_ecc= []#, blobs_dog_all_ecc = [], []
     ## Loop through each peak and only extract blobs given by the theoretical position
     cnt = -1
     big_image = np.zeros((framedim))
@@ -124,9 +130,10 @@ def process_images(filename, file_directory):
             continue     
         
         ### make one big image with patches
-        big_image[xtheo_min:xtheo_max, ytheo_min:ytheo_max] = thresh
+        big_image[xtheo_min:xtheo_max, ytheo_min:ytheo_max] = data_8bit_raw_bg[xtheo_min:xtheo_max, ytheo_min:ytheo_max]
+        # big_image[xtheo_min:xtheo_max, ytheo_min:ytheo_max] = thresh
 
-        minsigma, maxsigma = 10, 15
+        minsigma, maxsigma = 5, 15
         threshold_int = 0.01
         #Lapacian of gaussian
         blobs_log = blob_log(thresh, min_sigma=minsigma, max_sigma=maxsigma, 
@@ -134,47 +141,104 @@ def process_images(filename, file_directory):
         blobs_log[:, 2] = blobs_log[:, 2] * np.sqrt(2)
         
         #difference of gaussian
-        blobs_dog = blob_dog(thresh, min_sigma=minsigma, max_sigma=maxsigma, threshold=threshold_int)
-        blobs_dog[:, 2] = blobs_dog[:, 2] * np.sqrt(2)
+        # blobs_dog = blob_dog(thresh, min_sigma=minsigma, max_sigma=maxsigma, threshold=threshold_int)
+        # blobs_dog[:, 2] = blobs_dog[:, 2] * np.sqrt(2)
         
         blobs_log_all_x.append(xtheo_min+blobs_log[:,1])
-        blobs_dog_all_x.append(xtheo_min+blobs_dog[:,1])
+        # blobs_dog_all_x.append(xtheo_min+blobs_dog[:,1])
         
         blobs_log_all_y.append(ytheo_min+blobs_log[:,0])
-        blobs_dog_all_y.append(ytheo_min+blobs_dog[:,0])
+        # blobs_dog_all_y.append(ytheo_min+blobs_dog[:,0])
         
         blobs_log_all_ecc.append(blobs_log[:,2])
-        blobs_dog_all_ecc.append(blobs_dog[:,2])
-                
+        # blobs_dog_all_ecc.append(blobs_dog[:,2])
+        
+        
     blobs_log_all_x = np.concatenate(blobs_log_all_x, axis=0)
     blobs_log_all_y = np.concatenate(blobs_log_all_y, axis=0)
     blobs_log_all_ecc = np.concatenate(blobs_log_all_ecc, axis=0)
     
-    blobs_dog_all_x = np.concatenate(blobs_dog_all_x, axis=0)
-    blobs_dog_all_y = np.concatenate(blobs_dog_all_y, axis=0)
-    blobs_dog_all_ecc = np.concatenate(blobs_dog_all_ecc, axis=0)
+    # blobs_dog_all_x = np.concatenate(blobs_dog_all_x, axis=0)
+    # blobs_dog_all_y = np.concatenate(blobs_dog_all_y, axis=0)
+    # blobs_dog_all_ecc = np.concatenate(blobs_dog_all_ecc, axis=0)
 
     blobs_log = np.column_stack((blobs_log_all_x, blobs_log_all_y, blobs_log_all_ecc))
-    blobs_dog = np.column_stack((blobs_dog_all_x, blobs_dog_all_y, blobs_dog_all_ecc))            
+    # blobs_dog = np.column_stack((blobs_dog_all_x, blobs_dog_all_y, blobs_dog_all_ecc))            
     
     ## delete small circles
     val_del = np.sqrt(2)+0.1
     ind_del_log = np.where(blobs_log[:,2]<val_del)[0]
-    ind_del_dog = np.where(blobs_log[:,2]<val_del)[0]
+    # ind_del_dog = np.where(blobs_log[:,2]<val_del)[0]
     
     blobs_log = np.delete(blobs_log, ind_del_log, axis=0)
-    blobs_dog = np.delete(blobs_dog, ind_del_dog, axis=0)
+    # blobs_dog = np.delete(blobs_dog, ind_del_dog, axis=0)
+    
     # =============================================================================
     #   Write Cor file for lauetools      
     # =============================================================================
     nbpeaks, _ = blobs_log.shape    
-    peak_X = blobs_log[:,1]
-    peak_Y = blobs_log[:,0]
-    peak_I = blobs_log[:, 2] #np.ones(nbpeaks) * 255.
+    peak_X_nonfit = blobs_log[:,1]
+    peak_Y_nonfit = blobs_log[:,0]
+    peak_I_nonfit = blobs_log[:, 2] #np.ones(nbpeaks) * 255.
 
+    twicetheta, chi = Lgeo.calc_uflab(peak_X_nonfit, peak_Y_nonfit, detectorparameters,
+                                        returnAngles=1,
+                                        pixelsize=pixelsize,
+                                        kf_direction='Z>0')
+    dict_dp={}
+    dict_dp['kf_direction']='Z>0'
+    dict_dp['detectorparameters']=detectorparameters
+    dict_dp['detectordistance']=detectorparameters[0]
+    dict_dp['detectordiameter']=pixelsize*framedim[0]
+    dict_dp['pixelsize']=pixelsize
+    dict_dp['dim']=framedim
+    dict_dp['peakX']=peak_X_nonfit
+    dict_dp['peakY']=peak_Y_nonfit
+    dict_dp['intensity']=peak_I_nonfit
+    CCDcalib = {"CCDLabel":"sCMOS_16M",
+                "dd":detectorparameters[0], 
+                "xcen":detectorparameters[1], 
+                "ycen":detectorparameters[2], 
+                "xbet":detectorparameters[3], 
+                "xgam":detectorparameters[4],
+                "pixelsize": pixelsize}
+    path = os.path.normpath(filename)
+    IOLT.writefile_cor(file_directory+"//LOG_nofit//"+path.split(os.sep)[-1].split(".")[0], twicetheta, 
+                        chi, peak_X_nonfit, peak_Y_nonfit, peak_I_nonfit,
+                        param=CCDcalib, sortedexit=1)
     # =============================================================================
-    #     COR file
+    #     FIT PEAKS AGAIN WITH PEAK_SEARCH
     # =============================================================================
+    pixel_deviation = 15 #50
+    boxsize = 10
+    
+    type_of_function = "gaussian"
+    position_start = "max"
+
+    peaklist = np.vstack((peak_X_nonfit, peak_Y_nonfit)).T
+    peak_dataarray = RMCCD.fitoneimage_manypeaks(filename,
+                                                peaklist,
+                                                boxsize=boxsize,
+                                                stackimageindex=-1,
+                                                CCDLabel=CCDLabel,
+                                                dirname=None,
+                                                position_start=position_start,
+                                                type_of_function=type_of_function,
+                                                xtol=0.00001,
+                                                FitPixelDev=pixel_deviation,
+                                                Ipixmax=None,
+                                                MaxIntensity=65535, #e5,
+                                                MinIntensity=0,
+                                                PeakSizeRange=(0, 200),
+                                                verbose=0,
+                                                position_definition=1,
+                                                NumberMaxofFits=5000,
+                                                ComputeIpixmax=True,
+                                                use_data_corrected=None,
+                                                reject_negative_baseline=True)
+    
+    isorted = peak_dataarray[0]
+    peak_X, peak_Y, peak_I = isorted[:,0], isorted[:,1], isorted[:,2]
     twicetheta, chi = Lgeo.calc_uflab(peak_X, peak_Y, detectorparameters,
                                         returnAngles=1,
                                         pixelsize=pixelsize,
@@ -198,7 +262,77 @@ def process_images(filename, file_directory):
                 "xgam":detectorparameters[4],
                 "pixelsize": pixelsize}
     path = os.path.normpath(filename)
-    IOLT.writefile_cor(file_directory+"//"+path.split(os.sep)[-1].split(".")[0], twicetheta, 
+    IOLT.writefile_cor(file_directory+"//LT_LOG_ROIs//"+path.split(os.sep)[-1].split(".")[0], twicetheta, 
+                        chi, peak_X, peak_Y, peak_I,
+                        param=CCDcalib, sortedexit=1)
+
+
+    # =============================================================================
+    #     FIT PEAKS AGAIN WITH PEAK_SEARCH
+    # =============================================================================
+    ## specific intensity threshold based on beam condition
+    intensity_threshold = 100 #85
+    pixel_deviation = 15
+    boxsize = 10
+    
+    type_of_function = "gaussian"
+    position_start = "max"
+    
+    peaklist, Ipixmax = ImProc.LocalMaxima_ShiftArrays(big_image,
+                                                        framedim=framedim,
+                                                        IntensityThreshold=intensity_threshold,
+                                                        Saturation_value=65535,
+                                                        boxsize_for_probing_minimal_value_background=boxsize,  # 30
+                                                        pixeldistance_remove_duplicates=5,  # 25
+                                                        nb_of_shift=boxsize)  # 25
+    
+    peak_dataarray = RMCCD.fitoneimage_manypeaks(filename,
+                                                peaklist,
+                                                boxsize=boxsize,
+                                                stackimageindex=-1,
+                                                CCDLabel=CCDLabel,
+                                                dirname=None,
+                                                position_start=position_start,
+                                                type_of_function=type_of_function,
+                                                xtol=0.00001,
+                                                FitPixelDev=pixel_deviation,
+                                                Ipixmax=Ipixmax,
+                                                MaxIntensity=65535,
+                                                MinIntensity=0,
+                                                PeakSizeRange=(0, 200),
+                                                verbose=0,
+                                                position_definition=1,
+                                                NumberMaxofFits=5000,
+                                                ComputeIpixmax=True,
+                                                use_data_corrected=None,
+                                                reject_negative_baseline=True)
+    
+    isorted = peak_dataarray[0]
+    peak_X, peak_Y, peak_I = isorted[:,0], isorted[:,1], isorted[:,2]
+    twicetheta, chi = Lgeo.calc_uflab(peak_X, peak_Y, detectorparameters,
+                                        returnAngles=1,
+                                        pixelsize=pixelsize,
+                                        kf_direction='Z>0')
+    
+    dict_dp={}
+    dict_dp['kf_direction']='Z>0'
+    dict_dp['detectorparameters']=detectorparameters
+    dict_dp['detectordistance']=detectorparameters[0]
+    dict_dp['detectordiameter']=pixelsize*framedim[0]
+    dict_dp['pixelsize']=pixelsize
+    dict_dp['dim']=framedim
+    dict_dp['peakX']=peak_X
+    dict_dp['peakY']=peak_Y
+    dict_dp['intensity']=peak_I
+    CCDcalib = {"CCDLabel":"sCMOS_16M",
+                "dd":detectorparameters[0], 
+                "xcen":detectorparameters[1], 
+                "ycen":detectorparameters[2], 
+                "xbet":detectorparameters[3], 
+                "xgam":detectorparameters[4],
+                "pixelsize": pixelsize}
+    path = os.path.normpath(filename)
+    IOLT.writefile_cor(file_directory+"//LT_ROIs//"+path.split(os.sep)[-1].split(".")[0], twicetheta, 
                         chi, peak_X, peak_Y, peak_I,
                         param=CCDcalib, sortedexit=1)
 
@@ -298,7 +432,20 @@ if __name__ == "__main__":
     file_directory = folder_tiff + "\\cor_files"
     if not os.path.exists(file_directory):
         os.makedirs(file_directory)
-
+    
+    temp_ = folder_tiff + "\\cor_files\\LT_ROIs"
+    if not os.path.exists(temp_):
+        os.makedirs(temp_)
+        
+    temp_ = folder_tiff + "\\cor_files\\LT_LOG_ROIs"
+    if not os.path.exists(temp_):
+        os.makedirs(temp_)
+        
+    temp_ = folder_tiff + "\\cor_files\\LOG_nofit"
+    if not os.path.exists(temp_):
+        os.makedirs(temp_)
+    
+    
     experimental_prefix= "Zr3_1250_"
     format_file = "tif"
     
@@ -312,7 +459,7 @@ if __name__ == "__main__":
         for ino, filename in enumerate(list_of_files):
             if ino != 0:
                 continue
-            process_images(filename, file_directory)#♥, dat_flag, cor_flag)
+            process_images(filename, file_directory)#, dat_flag, cor_flag)
         
     else:
         args = zip(list_of_files, itertools.repeat(file_directory))#, itertools.repeat(dat_flag), itertools.repeat(cor_flag))
